// SystemInformation.cpp : Implementation of CSystemInformation
#include "stdafx.h"
#include "SystemInfo.h"
#include "SystemInformation.h"

/////////////////////////////////////////////////////////////////////////////
// CSystemInformation

CSystemInformation::CSystemInformation ()
{
	m_pOSInfo = NULL;
	m_pMouseInfo = NULL;
	m_pMemoryInfo = NULL;
	m_pHDiskInfo = NULL;
	m_pStorageInfo = NULL;
	m_pMultiMediaInfo = NULL;
	m_pCPUInfo = NULL;
	m_pMiscInfo = NULL;
}

HRESULT
CSystemInformation::FinalConstruct ()
{
	HRESULT hr;

	hr = CoCreateInstance (CLSID_OSInformation, 0, CLSCTX_INPROC_SERVER, IID_IOSInformation,
		(void **) &m_pOSInfo);
	if (FAILED (hr)) {
		return E_NOINTERFACE;
	}

	hr = CoCreateInstance (CLSID_MouseInformation, 0, CLSCTX_INPROC_SERVER,
		IID_IMouseInformation, (void **) &m_pMouseInfo);
	if (FAILED (hr)) {
		return E_NOINTERFACE;
	}

	hr = CoCreateInstance (CLSID_MemoryInformation, 0, CLSCTX_INPROC_SERVER,
		IID_IMemoryInformation, (void **) &m_pMemoryInfo);
	if (FAILED (hr)) {
		return E_NOINTERFACE;
	}

	hr = CoCreateInstance (CLSID_HDiskInformation, 0, CLSCTX_INPROC_SERVER,
		IID_IHDiskInformation, (void **) &m_pHDiskInfo);
	if (FAILED (hr)) {
		return E_NOINTERFACE;
	}

	hr = CoCreateInstance (CLSID_StorageMediaInformation, 0, CLSCTX_INPROC_SERVER,
		IID_IStorageMediaInformation, (void **) &m_pStorageInfo);
	if (FAILED (hr)) {
		return E_NOINTERFACE;
	}

	hr = CoCreateInstance (CLSID_MultiMediaInformation, 0, CLSCTX_INPROC_SERVER,
		IID_IMultiMediaInformation, (void **) &m_pMultiMediaInfo);
	if (FAILED (hr)) {
		return E_NOINTERFACE;
	}

	hr = CoCreateInstance (CLSID_CPUInformation, 0, CLSCTX_INPROC_SERVER,
		IID_ICPUInformation, (void **) &m_pCPUInfo);
	if (FAILED (hr)) {
		return E_NOINTERFACE;
	};

	hr = CoCreateInstance (CLSID_MiscInformation, 0, CLSCTX_INPROC_SERVER,
		IID_IMiscInformation, (void **) &m_pMiscInfo);
	if (FAILED (hr)) {
		return E_NOINTERFACE;
	};

	return S_OK;
}

void
CSystemInformation::FinalRelease ()
{
	if (m_pOSInfo != NULL) {
		m_pOSInfo->Release ();
	}

	if (m_pMouseInfo != NULL) {
		m_pMouseInfo->Release ();
	}

	if (m_pMemoryInfo != NULL) {
		m_pMemoryInfo->Release ();
	}

	if (m_pHDiskInfo != NULL) {
		m_pHDiskInfo->Release ();
	}

	if (m_pStorageInfo != NULL) {
		m_pStorageInfo->Release ();
	}

	if (m_pMultiMediaInfo != NULL) {
		m_pMultiMediaInfo->Release ();
	}

	if (m_pCPUInfo != NULL) {
		m_pCPUInfo->Release ();
	}

	if (m_pMiscInfo != NULL) {
		m_pMiscInfo->Release ();
	}
}

STDMETHODIMP CSystemInformation::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ISystemInformation
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CSystemInformation::GetOSInformation(BSTR *pbstrPlatform, BSTR *pbstrMinorVersion,
												  BSTR *pbstrServicePack, long *plBuildNumber)
{
	HRESULT hr = E_FAIL;

	hr = m_pOSInfo->GetOSInformation (plBuildNumber, pbstrPlatform, pbstrMinorVersion,
		pbstrServicePack);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CSystemInformation::GetMouseInformation(VARIANT_BOOL *pbInstalled, VARIANT_BOOL *pbButtonsSwapped,
													 long *plSpeed)
{
	HRESULT hr = E_FAIL;

	hr = m_pMouseInfo->GetMouseInformation (pbInstalled, pbButtonsSwapped, plSpeed);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CSystemInformation::GetCPUInformation(BSTR *pbstrVendor, BSTR *pbstrType,
	BSTR *pbstrArchitecture, BSTR *pbstrLevel, BSTR *pbstrRevision, long *plNumberOfProcessors,
	long *plSpeed, long *plPageSize, long *plMask)
{
	HRESULT hr = E_FAIL;

	hr = m_pCPUInfo->GetCPUInformation (pbstrVendor, pbstrType, pbstrArchitecture, pbstrLevel,
		pbstrRevision, plNumberOfProcessors, plSpeed, plPageSize, plMask);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CSystemInformation::GetHDiskInformation(long *plNumberOfPartitions,
	VARIANT *pbstrDriveLetterArr, VARIANT *pbBootableArr, VARIANT *pbstrTypeArr,
	VARIANT *plPartitionNumberArr, VARIANT *plLengthArr, VARIANT *plHiddenSectorsArr)
{
	HRESULT hr = E_FAIL;

	hr = m_pHDiskInfo->GetHDiskInformation(plNumberOfPartitions, pbstrDriveLetterArr,
		pbBootableArr, pbstrTypeArr, plPartitionNumberArr, plLengthArr, plHiddenSectorsArr);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CSystemInformation::GetMemoryInformation(long *plRAMInstalled,
	long *plAvailableMemory, long *plVirtualMemory, long *plMemoryLoad)
{
	HRESULT hr = E_FAIL;

	hr = m_pMemoryInfo->GetMemoryInformation (plRAMInstalled, plAvailableMemory, plVirtualMemory, plMemoryLoad);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CSystemInformation::GetStorageMediaInformation(long *plNumberOfDrives,
	VARIANT *pbReadyArr, VARIANT *pbstrDriveLetterArr, VARIANT *pbstrLabelArr,
	VARIANT *pbSupportLongNameArr, VARIANT *pbstrFileSystemArr, VARIANT *pbstrMediaTypeArr,
	VARIANT *plTotalSpaceArr, VARIANT *plFreeSpaceUserArr, VARIANT *plFreeSpaceTotalArr)
{
	HRESULT hr = E_FAIL;

	hr = m_pStorageInfo->GetStorageMediaInformation (plNumberOfDrives, pbReadyArr, pbstrDriveLetterArr,
		pbstrLabelArr, pbSupportLongNameArr, pbstrFileSystemArr, pbstrMediaTypeArr,
		plTotalSpaceArr, plFreeSpaceUserArr, plFreeSpaceTotalArr);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CSystemInformation::GetMultiMediaInformation(VARIANT_BOOL *pbIsInstalled,
	VARIANT_BOOL *pbHasVolCtrl, VARIANT_BOOL *pbHasSeparateLRVolCtrl, BSTR *pbstrProductName,
	BSTR *pbstrCompanyName)
{
	HRESULT hr = E_FAIL;

	hr = m_pMultiMediaInfo->GetMultiMediaInformation (pbIsInstalled, pbHasVolCtrl,
		pbHasSeparateLRVolCtrl, pbstrProductName, pbstrCompanyName);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CSystemInformation::GetComputerName(BSTR *pbstrComputerName)
{
	HRESULT hr = E_FAIL;

	hr = m_pMiscInfo->get_ComputerName(pbstrComputerName);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CSystemInformation::GetUserName(BSTR *pbstrUserName)
{
	HRESULT hr = E_FAIL;

	hr = m_pMiscInfo->get_UserName(pbstrUserName);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}

STDMETHODIMP CSystemInformation::GetLocalLanguage(BSTR *pbstrLocalLanguage)
{
	HRESULT hr = E_FAIL;

	hr = m_pMiscInfo->get_Language (pbstrLocalLanguage);
	if (FAILED (hr)) {
		return E_FAIL;
	}

	return S_OK;
}
