// OSInformation.cpp : Implementation of COSInformation
#include "stdafx.h"
#include "SystemOS.h"
#include "OSInformation.h"

/////////////////////////////////////////////////////////////////////////////
// COSInformation

STDMETHODIMP COSInformation::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IOSInformation
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP COSInformation::get_BuildNumber(long *pVal)
{
	HRESULT hRes = S_OK;
	if ((hRes = GetInformation ()) != S_OK) {
		return hRes;
	}

	*pVal = m_sBuildNumber;

	return S_OK;
}

STDMETHODIMP COSInformation::get_IsWin95(BOOL *pVal)
{
	HRESULT hRes = S_OK;
	if ((hRes = GetInformation ()) != S_OK) {
		return hRes;
	}

	*pVal = m_bIsWin95;

	return S_OK;
}

STDMETHODIMP COSInformation::get_Platform(BSTR *pVal)
{
	HRESULT hRes = S_OK;
	if ((hRes = GetInformation ()) != S_OK) {
		return hRes;
	}

	*pVal = m_bstrPlatform.Copy ();

	return S_OK;
}

STDMETHODIMP COSInformation::get_MinorVersion(BSTR *pVal)
{
	HRESULT hRes = S_OK;
	if ((hRes = GetInformation ()) != S_OK) {
		return hRes;
	}

	*pVal = m_bstrMinorVersion.Copy ();

	return S_OK;
}

STDMETHODIMP COSInformation::get_ServicePack(BSTR *pVal)
{
	HRESULT hRes = S_OK;
	if ((hRes = GetInformation ()) != S_OK) {
		return hRes;
	}

	*pVal = m_bstrServicePack.Copy ();

	return S_OK;
}

STDMETHODIMP COSInformation::GetOSInformation(long *plBuildNumber, BSTR *pbstrPlatform,
											  BSTR *pbstrMinorVersion,
											  BSTR *pbstrServicePack)
{
	HRESULT hRes = S_OK;
	if ((hRes = GetInformation ()) != S_OK) {
		return hRes;
	}

	*plBuildNumber = m_sBuildNumber;
	*pbstrPlatform = m_bstrPlatform.Copy ();
	*pbstrMinorVersion = m_bstrMinorVersion.Copy ();
	*pbstrServicePack = m_bstrServicePack.Copy ();

	return S_OK;
}

HRESULT
COSInformation::GetInformation ()
{
	int stat = 0;
	UINT resourceID = 0;
	TCHAR data [64];
	DWORD dataSize, dwError;
	OSVERSIONINFO versionInfo;
	HKEY hKey;
	LONG result;
	LPTSTR lpErrDesc = NULL;

	// If we already have generated the information, then there is no need to do it again.

	if (m_bInfoGenerated) {
		return S_OK;
	}
	
	// set the size of OSVERSIONINFO, before calling the function

	versionInfo.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);

	// Get the version information

	if (!::GetVersionEx (&versionInfo)) {
		dwError = GetLastError ();
		MakeErrorDesc (dwError, lpErrDesc);
		return Error (lpErrDesc, IID_IOSInformation, E_FAIL);
	}

	// If NT ,then check if its server or workstation.

	char szBuffer[128];

	if (versionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT) {
		// there is no direct way of telling from ODVERSIONINFO thats is it 
		// workstation or server version.
		// There we need to check in the registry.

		m_bstrPlatform.LoadString (IDS_WINDOWS_NT);

		dataSize = sizeof (data);
		
		result = ::RegOpenKeyEx (HKEY_LOCAL_MACHINE,
			"System\\CurrentControlSet\\Control\\ProductOptions",
			0, KEY_QUERY_VALUE, &hKey);

		// If there is error in opening the registry key, return

		if (result != ERROR_SUCCESS) {
			dwError = GetLastError ();
			MakeErrorDesc (dwError, lpErrDesc);
			return Error (lpErrDesc, IID_IOSInformation, E_FAIL);
		}

		result = ::RegQueryValueEx (hKey, _T("ProductType"), NULL, NULL, (LPBYTE) data,
			&dataSize);

		// Make sure to close the reg key

		RegCloseKey (hKey);

		if (result != ERROR_SUCCESS) {
			dwError = GetLastError ();
			MakeErrorDesc (dwError, lpErrDesc);
			return Error (lpErrDesc, IID_IOSInformation, E_FAIL);
		}

		// Check what string has been returned

		if (lstrcmpi (data, "WinNT") == 0) {
			resourceID = IDS_NT_WORKSTATION;
		}
		else if (lstrcmpi (data, "ServerNT") == 0) {
			resourceID = IDS_NT_SERVER;
		}
		else {
			resourceID = IDS_NT_DOMAIN_CONTROLLER;
		}

		m_bstrPlatform.LoadString (resourceID);

		// Check the version number

		if (versionInfo.dwMajorVersion == 3 || versionInfo.dwMinorVersion == 51) {
			resourceID = IDS_NT_351;
		}
		else {
			resourceID = IDS_NT_40;
		}

		m_bstrMinorVersion.LoadString (resourceID);

		// Get the build number.

		m_sBuildNumber = versionInfo.dwBuildNumber;
	}
	else if (versionInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) {
		resourceID = IDS_WINDOWS_95;
		m_bIsWin95 = TRUE;

		// Windows 98 test

		if ((versionInfo.dwMajorVersion > 4) || ((versionInfo.dwMajorVersion == 4) &&
			(versionInfo.dwMinorVersion > 0))) {
			resourceID = IDS_WINDOWS_98;
		}

		if (LoadString (_Module.GetResourceInstance(), resourceID, szBuffer, 
			sizeof (szBuffer))) {
			m_bstrPlatform = szBuffer;
		}
		
		// For win95, build number is low order word
		
		m_sBuildNumber = (DWORD)(LOBYTE(LOWORD(versionInfo.dwBuildNumber)));
	}
	else if (versionInfo.dwPlatformId == VER_PLATFORM_WIN32s) {
		if (LoadString (_Module.GetResourceInstance(), IDS_WINDOWS_32S, szBuffer, 
			sizeof (szBuffer))) {
			m_bstrPlatform = szBuffer;
		}
	}
	else {
		if (LoadString (_Module.GetResourceInstance(), IDS_WINDOWS_31, szBuffer, 
			sizeof (szBuffer))) {
			m_bstrPlatform = szBuffer;
		}
	}

	// Get service pack information.

	m_bstrServicePack = versionInfo.szCSDVersion;

	m_bInfoGenerated = true;

	return S_OK;
}

void
COSInformation::MakeErrorDesc (DWORD errCode, LPVOID lpErrorDesc)
{
	FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS, NULL, errCode, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR) &lpErrorDesc, 0, NULL);
}
